import 'notification.m.dart';
import 'appletAPI.m.dart';
import 'liveConsultation.m.dart';

import 'package:fis_jsonrpc/utils.dart';

enum BookingTypeEnum {
	Personal,
	Group,
}

enum BookingStatusEnum {
	UnSubmitted,
	Booked,
	Canceled,
}

class HealthExamPersonDTO {
	String? name;
	PatientGenderEnum gender;
	String? identityCard;
	String? phone;

	HealthExamPersonDTO({
		this.name,
		this.gender = PatientGenderEnum.NotFilled,
		this.identityCard,
		this.phone,
	});

	factory HealthExamPersonDTO.fromJson(Map<String, dynamic> map) {
		return HealthExamPersonDTO( 
			name: map['Name'],
			gender: PatientGenderEnum.values.firstWhere((e) => e.index == map['Gender']),
			identityCard: map['IdentityCard'],
			phone: map['Phone'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if (name != null) {
			map['Name'] = name;
		}
		map['Gender'] = gender.index;
		if (identityCard != null) {
			map['IdentityCard'] = identityCard;
		}
		if (phone != null) {
			map['Phone'] = phone;
		}
		return map;
	}
}

class HealthExamItemDTO {
	String? name;
	String? content;

	HealthExamItemDTO({
		this.name,
		this.content,
	});

	factory HealthExamItemDTO.fromJson(Map<String, dynamic> map) {
		return HealthExamItemDTO( 
			name: map['Name'],
			content: map['Content'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if (name != null) {
			map['Name'] = name;
		}
		if (content != null) {
			map['Content'] = content;
		}
		return map;
	}
}

class HealthExamBookingDTO extends BaseDTO{
	String? code;
	BookingTypeEnum bookingType;
	String? name;
	String? location;
	DateTime? startDate;
	DateTime? endDate;
	String? subject;
	String? description;
	String? initiatorCode;
	String? orgCode;
	BookingStatusEnum status;
	List<HealthExamPersonDTO>? persons;
	List<HealthExamItemDTO>? examItems;

	HealthExamBookingDTO({
		this.code,
		this.bookingType = BookingTypeEnum.Personal,
		this.name,
		this.location,
		this.startDate,
		this.endDate,
		this.subject,
		this.description,
		this.initiatorCode,
		this.orgCode,
		this.status = BookingStatusEnum.UnSubmitted,
		this.persons,
		this.examItems,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory HealthExamBookingDTO.fromJson(Map<String, dynamic> map) {
		return HealthExamBookingDTO( 
			code: map['Code'],
			bookingType: BookingTypeEnum.values.firstWhere((e) => e.index == map['BookingType']),
			name: map['Name'],
			location: map['Location'],
			startDate: map['StartDate'] != null ? DateTime.parse(map['StartDate']) : null,
			endDate: map['EndDate'] != null ? DateTime.parse(map['EndDate']) : null,
			subject: map['Subject'],
			description: map['Description'],
			initiatorCode: map['InitiatorCode'],
			orgCode: map['OrgCode'],
			status: BookingStatusEnum.values.firstWhere((e) => e.index == map['Status']),
			persons: map['Persons'] != null ? (map['Persons'] as List).map((e)=>HealthExamPersonDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			examItems: map['ExamItems'] != null ? (map['ExamItems'] as List).map((e)=>HealthExamItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if (code != null)
			map['Code'] = code;
		map['BookingType'] = bookingType.index;
		if (name != null)
			map['Name'] = name;
		if (location != null)
			map['Location'] = location;
		if (startDate != null)
			map['StartDate'] = JsonRpcUtils.dateFormat(startDate!);
		if (endDate != null)
			map['EndDate'] = JsonRpcUtils.dateFormat(endDate!);
		if (subject != null)
			map['Subject'] = subject;
		if (description != null)
			map['Description'] = description;
		if (initiatorCode != null)
			map['InitiatorCode'] = initiatorCode;
		if (orgCode != null)
			map['OrgCode'] = orgCode;
		map['Status'] = status.index;
		if (persons != null)
			map['Persons'] = persons;
		if (examItems != null)
			map['ExamItems'] = examItems;
		return map;
	}
}

class GetHealthExamBookingPageRequest extends PageRequest{
	String? keyword;
	List<String>? orgCodes;

	GetHealthExamBookingPageRequest({
		this.keyword,
		this.orgCodes,
		int pageIndex = 0,
		int pageSize = 0,
		String? token,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			token: token,
		);

	factory GetHealthExamBookingPageRequest.fromJson(Map<String, dynamic> map) {
		return GetHealthExamBookingPageRequest( 
			keyword: map['Keyword'],
			orgCodes: map['OrgCodes']?.cast<String>().toList(),
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if (keyword != null)
			map['Keyword'] = keyword;
		if (orgCodes != null)
			map['OrgCodes'] = orgCodes;
		return map;
	}
}

class SaveHealthExamBookingRequest extends TokenRequest{
	String? code;
	BookingTypeEnum bookingType;
	String? name;
	String? location;
	DateTime? startDate;
	DateTime? endDate;
	String? subject;
	String? description;
	String? initiatorCode;
	String? orgCode;
	BookingStatusEnum status;
	List<HealthExamPersonDTO>? persons;
	List<HealthExamItemDTO>? examItems;

	SaveHealthExamBookingRequest({
		this.code,
		this.bookingType = BookingTypeEnum.Personal,
		this.name,
		this.location,
		this.startDate,
		this.endDate,
		this.subject,
		this.description,
		this.initiatorCode,
		this.orgCode,
		this.status = BookingStatusEnum.UnSubmitted,
		this.persons,
		this.examItems,
		String? token,
	}) : super(
			token: token,
		);

	factory SaveHealthExamBookingRequest.fromJson(Map<String, dynamic> map) {
		return SaveHealthExamBookingRequest( 
			code: map['Code'],
			bookingType: BookingTypeEnum.values.firstWhere((e) => e.index == map['BookingType']),
			name: map['Name'],
			location: map['Location'],
			startDate: map['StartDate'] != null ? DateTime.parse(map['StartDate']) : null,
			endDate: map['EndDate'] != null ? DateTime.parse(map['EndDate']) : null,
			subject: map['Subject'],
			description: map['Description'],
			initiatorCode: map['InitiatorCode'],
			orgCode: map['OrgCode'],
			status: BookingStatusEnum.values.firstWhere((e) => e.index == map['Status']),
			persons: map['Persons'] != null ? (map['Persons'] as List).map((e)=>HealthExamPersonDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			examItems: map['ExamItems'] != null ? (map['ExamItems'] as List).map((e)=>HealthExamItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if (code != null)
			map['Code'] = code;
		map['BookingType'] = bookingType.index;
		if (name != null)
			map['Name'] = name;
		if (location != null)
			map['Location'] = location;
		if (startDate != null)
			map['StartDate'] = JsonRpcUtils.dateFormat(startDate!);
		if (endDate != null)
			map['EndDate'] = JsonRpcUtils.dateFormat(endDate!);
		if (subject != null)
			map['Subject'] = subject;
		if (description != null)
			map['Description'] = description;
		if (initiatorCode != null)
			map['InitiatorCode'] = initiatorCode;
		if (orgCode != null)
			map['OrgCode'] = orgCode;
		map['Status'] = status.index;
		if (persons != null)
			map['Persons'] = persons;
		if (examItems != null)
			map['ExamItems'] = examItems;
		return map;
	}
}

class GetHealthExamBookingRequest extends TokenRequest{
	String? code;

	GetHealthExamBookingRequest({
		this.code,
		String? token,
	}) : super(
			token: token,
		);

	factory GetHealthExamBookingRequest.fromJson(Map<String, dynamic> map) {
		return GetHealthExamBookingRequest( 
			code: map['Code'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if (code != null)
			map['Code'] = code;
		return map;
	}
}

class DeleteHealthExamBookingRequest extends TokenRequest{
	String? code;

	DeleteHealthExamBookingRequest({
		this.code,
		String? token,
	}) : super(
			token: token,
		);

	factory DeleteHealthExamBookingRequest.fromJson(Map<String, dynamic> map) {
		return DeleteHealthExamBookingRequest( 
			code: map['Code'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if (code != null)
			map['Code'] = code;
		return map;
	}
}

class SignUpHealthExamBookingRequest extends TokenRequest{
	String? code;
	String? name;
	PatientGenderEnum gender;
	String? identityCard;
	String? phone;

	SignUpHealthExamBookingRequest({
		this.code,
		this.name,
		this.gender = PatientGenderEnum.NotFilled,
		this.identityCard,
		this.phone,
		String? token,
	}) : super(
			token: token,
		);

	factory SignUpHealthExamBookingRequest.fromJson(Map<String, dynamic> map) {
		return SignUpHealthExamBookingRequest( 
			code: map['Code'],
			name: map['Name'],
			gender: PatientGenderEnum.values.firstWhere((e) => e.index == map['Gender']),
			identityCard: map['IdentityCard'],
			phone: map['Phone'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if (code != null)
			map['Code'] = code;
		if (name != null)
			map['Name'] = name;
		map['Gender'] = gender.index;
		if (identityCard != null)
			map['IdentityCard'] = identityCard;
		if (phone != null)
			map['Phone'] = phone;
		return map;
	}
}


